/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.framework.server;

import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;

import sync4j.framework.logging.Sync4jLogger;

import java.util.logging.Logger;
import java.util.logging.Level;

import sync4j.framework.security.Sync4jPrincipal;

import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Represent the mapping between the LUID from the client Items and
 * the GUID from the server Items.
 *
 * @version $Id: ClientMapping.java,v 1.9 2006/02/07 09:56:57 luigiafassina Exp $
 */
public class ClientMapping {

    //Contain GUID -> LUID for each client
    private HashMap clientMapping = new HashMap();

    //Contain the principal of the client device
    private Sync4jPrincipal principal = null;

    // the requested database
    private String dbURI;

    //When the Client mapping are modified or added
    private boolean modified = false;
    private ArrayList modifiedKeys = new ArrayList();

    //When entries in the mapping are deleted
    private boolean deleted = false;
    private ArrayList deletedKeys = new ArrayList();

    //Sync4j Logging faciiy
    //Transient keyword will disable serialisation for this member
    private transient Logger log = Sync4jLogger.getLogger();

    // ------------------------------------------------------------- Contructors

    /**
     * Construct a client mapping to a device Id the mapping must be populate
     * with data from the persistence store by calling
     * PersistenceStoreManager.read.
     *
     */
    public ClientMapping(Sync4jPrincipal principal, String dbURI) {
        this.principal = principal;
        this.dbURI     = dbURI    ;
    }

    // ---------------------------------------------------------- Public methods

    /**
     * @return true if the mapping where modified or added since initialization
     */
    public boolean isModified() {
        return modified;
    }

    /**
     * @return true if item where deleted since the initialisation
     */
    public boolean isDeleted() {
        return deleted;
    }

    /**
     * Initialize the client mapping with data from the Persistence Store
     * @param mapping A structure that contain GUID -> LUID
     */
    public void initializeFromMapping(Map mapping) {
        resetMapping();
        clientMapping.putAll(mapping);
    }

    /**
     * Get the deleted LUIDs from the client mapping
     * @return a String[] of LUIDs deleted
     */
    public String[] getDeletedLuids() {
        if (deleted) {
            return (String[])deletedKeys.toArray(new String[deletedKeys.size()]);
        }

        return new String[0];
    }

    /**
     * Get the modified LUIDs from the client mapping
     * @return a String[] of LUIDs modified
     */
    public String[] getModifiedLuids() {
        if (modified) {
            return (String[])modifiedKeys.toArray(new String[modifiedKeys.size()]);
        }

        return new String[0];
    }

    /**
     * Get the current mapping.
     *
     * @return the current mapping as a Map
     */
    public Map getMapping() {
        Map ret = new HashMap();
        ret.putAll(clientMapping);
        return ret;
    }

    /**
     * Get the Client Device Id that correspond to that Client
     * @return The Device Id
     */
    public String getClientDeviceId() {
        return principal.getDeviceId();
    }

    /**
     * Get the principal
     *
     * @return the principal
     */
    public Sync4jPrincipal getPrincipal() {
        return principal;
    }

    /**
     * Get the database uri
     *
     * @return <i>dbURI</i>
     */
    public String getDbURI() {
        return dbURI;
    }

    /**
     * Add a mapping between a GUID and a LUID from the server
     * @param guid The GUID of the server item
     * @param luid The LUID of the client item
     */
    private void addMapping(String guid, String luid) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Adding mapping LUID-GUID " + luid + "-" + guid);
        }

        //
        // if the same guid was already mapped to a luid, we have to remmeber 
        // the former one and remove it from the list of updated/deleted luids
        //
        String oldLuid = (String)clientMapping.put(guid, luid);
        modifiedKeys.add(luid); modified = true;
        
        if (oldLuid != null) {
            removeDeletedKey(oldLuid);
            removeModifiedKey(oldLuid);
        }

        //
        // If we add an entry that was considered deleted, 
        // remove it from the deleted entries.
        //
        removeDeletedKey(luid);
    }

    /**
     * Get the mapped GUID for the given LUID.
     * @param luid The LUID of the client item
     * @return The mapped value for the key in input
     */
    public String getMappedValueForLuid(String luid) {
        String result = null;

        if (clientMapping.containsValue(luid)) {
            java.util.Set keys = clientMapping.keySet();
            java.util.Iterator itKeys = keys.iterator();
            while (itKeys.hasNext()) {
                String key = (String)itKeys.next();
                String value = (String)clientMapping.get(key);
                if (value.equals(luid)) {
                    return key;
                }
            }
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("No mapping found for LUID: " + luid);
        }
        return result;
    }

    /**
     * Get the mapped LUID key for the given GUID value.
     * @param guid The GUID of the client item
     * @return The mapped value for the key in input
     */
    public String getMappedValueForGuid(String guid) {
        String result = null;
        if ((result = (String) clientMapping.get(guid)) == null) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("No mapping found for GUID: " + guid);
            }
        }
        return result;
    }

    /**
     * Remove a mapped values from the cache given a LUID.
     * @param luid The LUID for the item from the client
     */
    public void removeMappedValuesForLuid(String luid) {
        String guid = getMappedValueForLuid(luid);
        if (guid != null) {
            clientMapping.remove(guid);
            addDeletedKey(luid);
        }
    }

    /**
     * Remove a mapped values from the cache given a GUID
     * @param guid The GUID for the item from the client
     */
    public void removeMappedValuesForGuid(String guid) {
        String luid = getMappedValueForGuid(guid);
        if (luid != null) {
            clientMapping.remove(guid);
            addDeletedKey(luid);
        }
    }

    /**
     * Updates a key value mapping. If the mapping does not exist, it calls
     * <code>addMapping(guid, luid)</code>, otherwise the existing mapping is
     * updated.
     *
     * @param guid The GUID value from the server item
     * @param luid The LUID value from the client item
     */
    public void updateMapping(String guid, String luid) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Updating mapping LUID-GUID " + luid + "-" + guid);
        }

        //
        // If new, just call addMapping
        //
        if (clientMapping.containsKey(guid) == false) {
            addMapping(guid, luid);
            return;
        }

        //
        // if the same guid was already mapped to a luid, we have to remmeber 
        // the former one and remove it from the list of updated/deleted luids
        //
        String oldLuid = (String)clientMapping.put(guid, luid);
        modifiedKeys.add(luid); modified = true;
        
        if (oldLuid != null) {
            removeDeletedKey(oldLuid);
            removeModifiedKey(oldLuid);
        }
        
        //
        // If we add an entry that was considered deleted, 
        // remove it from the deleted entries.
        //
        removeDeletedKey(luid);
    }

    /**
     * Clears these mappings moving the existing and new mappings to deleted.
     * This is used for example to re-initialize the LUID-GUID mapping for a
     * slow sync.
     */
    public void clearMappings() {

        deletedKeys.addAll(clientMapping.values());
        deletedKeys.addAll(modifiedKeys);

        clientMapping.clear();

        modified = false;
        deleted = true;
    }

    public String toString() {
         ToStringBuilder sb = new ToStringBuilder(this);

         sb.append("clientMapping", clientMapping);
         sb.append("modifiedKeys" , modifiedKeys );
         sb.append("deletedKeys"  , deletedKeys  );

         return sb.toString();
    }
    
    /**
     * Resets the modified (updated/deleted) uids
     */
    public void resetModifiedKeys() {
        //If we have modified item to the map
        if (modified) {
            modifiedKeys.clear();
            modified = false;
        }
        //If we have deleted item to the map
        if (deleted) {
            deletedKeys.clear();
            deleted = false;
        }
    }

    // --------------------------------------------------------- Private methods

    /**
     * Reset the content of the Client mapping HashMap and the state
     * of the current mapping.
     */
    private void resetMapping() {
        //We clean the map
        if (!clientMapping.isEmpty()) {
            clientMapping.clear();
        }
        
        resetModifiedKeys();
    }

    /**
     * Remove LUID from the deleted entries
     * @param luid The LUID for the item from the client
     */
    private void removeDeletedKey(String luid) {
        if (deleted && deletedKeys.contains(luid)) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Removing deleted LUID: " + luid);
            }
            deletedKeys.remove(luid);
            if (deletedKeys.size() == 0) {
                deleted = false;
            }
        }
    }

    /**
     * Remove LUID from the modified entries
     * @param luid The LUID for the item from the client
     */
    private void removeModifiedKey(String luid) {
        if (modified && modifiedKeys.contains(luid)) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Removing deleted LUID: " + luid);
            }
            modifiedKeys.remove(luid);
            if (modifiedKeys.size() == 0) {
                modified = false;
            }
        }
    }

    /**
     * Add LUID into the deleted keys.
     * @param luid The LUID for the item from the client
     */
    private void addDeletedKey(String luid) {
        removeModifiedKey(luid);
        deleted = true;
        deletedKeys.add(luid);
    }
}
