/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
package sync4j.server.config;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;

import sync4j.framework.config.ConfigurationException;
import sync4j.framework.tools.beans.BeanFactory;

import org.apache.commons.lang.StringUtils;
import sync4j.framework.tools.beans.BeanException;

/**
 * Utility class for configuration server beans handling
 *
 * @author  Stefano Fornari @ Funambol
 * @version $Id: ConfigTools.java,v 1.2 2006/02/07 09:56:58 luigiafassina Exp $
 */
public class ConfigTools {
    
    // ------------------------------------------------------------ Private data
    
    static private ConfigCache cache;
    
    // ---------------------------------------------------------- Public methods
    /**
     * It is supposed that the value of the parameter is a bean name. It returns
     * that bean, created with <i>BeanFactory.getBeanInstance()</i>.
     *
     * @param configLoader class loader to be used to load classes and 
     *        configuration objects
     * @param name the name of the bean
     *
     * @return an instance of the class specified by the parameter
     *
     * @throws ConfigurationException in case of errors
     */
    public static Object getNewBeanInstance(ClassLoader configLoader, String name)
    throws ConfigurationException {
        return cache.getNewBeanInstance(configLoader, name);
    }
    
    /**
     * It is supposed that the value of the parameter is a bean name. It works 
     * like getBeanInstance, but it tries to return always the same instance
     * unless the config file changes.
     *
     * @param configLoader class loader to be used to load classes and 
     *        configuration objects
     * @param name the name of the bean
     *
     * @return an instance of the class specified by the parameter
     *
     * @throws ConfigurationException in case of errors
     */
    public static Object getBeanInstance(ClassLoader configLoader, String name)
    throws ConfigurationException {
        return cache.getBeanInstance(configLoader, name);
    }

    /**
     * Create and return an array of bean as specified by a list of bean names.
     *
     * @param configLoader class loader to be used to load classes and 
     *        configuration objects
     * @param names the name of the anme
     * @param separator the separator character
     *
     */
    public static Object[] getBeanArray(ClassLoader configLoader, String names, char separator)
    throws ConfigurationException {
        String[] values = StringUtils.split(names, String.valueOf(separator));

        Object[] ret = new Object[values.length];

        int i = 0;
        try {
            for (; i<ret.length; ++i) {
                ret[i] = BeanFactory.getBeanInstance(configLoader, values[i]);
            }
        } catch (Exception e) {
            throw new ConfigurationException ( "Error in creating the bean "
                                             + values[i]
                                             , e                           );
        }

        return ret;
    }
    
    /**
     * Checks if the given string is an accepted URL (starting with http://,
     * https:// or file://). If yes, a new URL object representing the given
     * url is returned; otherwise, the given string is considered a file name
     * and a new URL is obtained calling File.toURL().
     *
     * @param s the string to check
     *
     * @return the corresponding URL if the string represents a URL or the
     *         fixed URL if the string is a pathname/filename
     *
     * @throws MalformedURLException
     */
    public static URL fixURL(final String s)
    throws MalformedURLException {

        try {
            return new URL(s);
        } catch (MalformedURLException e) {
            //
            // This is not a URL, let's consider it just a file
            //
        }

        try {
            return new File(new File(s).getCanonicalPath()).toURL();
        } catch (IOException e) {
            throw new MalformedURLException("Unable to convert" + s + " to a URL");
        }
    }
    
    /**
     * Saves the given server bean and invalidates the cache entry if any.
     *
     * @param beanName the complete beanName
     * @param obj the bean instance
     *
     *
     * @throws BeanException in case of instantiation error
     */
    public static void saveBeanInstance(String beanName, File beanFile, Object o) 
    throws BeanException {
        BeanFactory.saveBeanInstance(o, beanFile);
        cache.invalidate(beanName);
    }
    
    static {
        cache = new ConfigCache();
    }

}
