/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.server.syncbean;

import javax.ejb.*;
import javax.naming.*;

import javax.rmi.PortableRemoteObject;

import java.io.*;
import java.util.Map;
import java.util.logging.Logger;
import java.util.logging.Level;

import sync4j.framework.core.*;
import sync4j.framework.server.SyncResponse;
import sync4j.framework.logging.Sync4jLogger;
import sync4j.framework.config.ConfigClassLoader;
import sync4j.framework.config.ConfigurationException;
import sync4j.framework.protocol.ProtocolException;

import sync4j.framework.core.Sync4jException;
import sync4j.framework.core.RepresentationException;
import sync4j.framework.server.session.SessionExpiredException;
import sync4j.framework.server.error.*;

import sync4j.server.config.Configuration;
import sync4j.server.engine.SyncAdapter;
import sync4j.server.session.SyncSessionHandler;

import sync4j.server.admin.*;
import sync4j.server.admin.ejb.*;

/**
 *  This is the session enterprise java bean that handles a synchronization
 *  request. It is designed to be a stateful session bean so that the state
 *  and the session management are on charge of the application server.
 *
 *  LOG NAME: sync4j.server
 *
 *  @author Stefano Fornari @ Funambol
 *  @author Luigia Fassina @ Funambol
 *
 *  @version $Id: SyncBean.java,v 1.62 2006/02/07 09:56:58 luigiafassina Exp $
 *
 */
public class SyncBean
implements javax.ejb.SessionBean {

    // ------------------------------------------------------- Private constants
    
    private static final String ENV_ADMIN_NAME
        = "java:comp/env/ejb/LocalAdminBean";

    // ------------------------------------------------------------ Private data

    private transient Logger log = null;

    private SessionContext sessionContext       = null;

    private SyncSessionHandler sessionHandler = null;

    private Configuration config                = null;

    private String sessionId                    = null;

    private SyncAdapter syncAdapter             = null;

    // ------------------------------------------------------------- EJB methods

    public void ejbCreate(String sessionId) throws CreateException {

        log = Sync4jLogger.getLogger("server");

        loadAdminBean();

        try {
            syncAdapter = new SyncAdapter(config);
            syncAdapter.beginSync(sessionId);
        } catch (Exception e) {
            log.throwing("SyncBean", "ejbCreate", e);
            throw new CreateException( "Error "
                                     + e.getClass().getName()
                                     + " creating the SyncBean: "
                                     + e.getMessage()
                                     );
        }
        this.sessionId = sessionId;
    }

    /**
     * Creates an instance of AdminBean
     */
    private void loadAdminBean() throws CreateException {

        try {

            InitialContext ctx = new InitialContext();

            Object obj = ctx.lookup(ENV_ADMIN_NAME);
            AdminHomeLocal home = 
                (AdminHomeLocal)PortableRemoteObject.narrow(obj, AdminHomeLocal.class);
            AdminLocal admin = home.create();

            config = admin.getConfig();
            
        } catch (NamingException e) {
            e.printStackTrace();
            log.throwing("SyncBean", "loadAdminBean", e);
            throw new CreateException( "Error "
                                     + e.getClass().getName()
                                     + " creating the AdminBean: "
                                     + e.getMessage()
                                     );
        } catch (Exception e) {
            e.printStackTrace();
            log.throwing("SyncBean", "loadAdminBean", e);
            throw new CreateException( "Error "
                                     + e.getClass().getName()
                                     + " creating the AdminBean: "
                                     + e.getMessage()
                                     );
        }
    }


    /**
     * When the session terminates this object is removed. This can happen
     * because the synchronization process has been completed or because the
     * session has expired. If the session completes its job, the synchro-
     * nization can be committed.
     *
     */
    public void ejbRemove() {
        syncAdapter.endSync();
    }

    public void ejbActivate() {
        log = Sync4jLogger.getLogger();
    }

    public void ejbPassivate() {
        log = null;
    }

    public void setSessionContext (SessionContext sessionContext)
    throws EJBException {
        this.sessionContext = sessionContext;
    }

    /** Processes an incoming XML message.
     *
     * @param msg the SyncML request as an array of bytes
     * @param parameters SyncML request parameters
     * @param headers SyncML request headers
     *
     * @return the SyncML response as a <i>ISyncResponse</i> object
     *
     * @throws ServerException in case of a server error
     *
     */
    public SyncResponse processXMLMessage(final byte[] msg       , 
                                          final Map    parameters , 
                                          final Map    headers    )
    throws ServerException {
        return (SyncResponse) syncAdapter.processXMLMessage(msg, parameters, headers);
    }
    
    /** Processes an incoming WBXML message.
     *
     * @param msg the SyncML request (WBXML encoded) as an array of bytes
     * @param parameters SyncML request parameters
     * @param headers SyncML request headers
     *
     * @return the SyncML response as a <i>ISyncResponse</i> object
     *
     * @throws ServerException in case of a server error
     *
     */
    public SyncResponse processWBXMLMessage(final byte[] msg       , 
                                            final Map    parameters , 
                                            final Map    headers    )
    throws ServerException {
        return (SyncResponse) syncAdapter.processWBXMLMessage(msg, parameters, headers);
    }

    /**
     * Used to process a status information as needed by the client object (i.e.
     * errors or success).
     *
     * @param statusCode the status code
     * @param info additional informational message
     *
     * @see sync4j.framework.core.StatusCode for valid status codes.
     */
    public SyncResponse processStatusCode(int statusCode, String info){
        return syncAdapter.processStatusCode(statusCode, info);
    }
}
